from enum import IntEnum


class ReportReason(IntEnum):
    CHEATING_WALLHACK = 0
    CHEATING_AIMBOT = 1
    CHEATING_SPEEDHACK = 2
    RACISM_DISCRIMINATION_TEXT = 3
    RACISM_DISCRIMINATION_VOICE = 4
    SPAMMING_TEXT = 5
    SPAMMING_VOICE = 6
    BAD_LANGUAGE_TEXT = 7
    BAD_LANGUAGE_VOICE = 8
    GRIEFING = 9
    EXPLOITING = 10
    OTHER_TOXIC_BEHAVIOUR = 11
    USER_PROFILE_NAME_PICTURE = 12
