from enum import IntEnum


class WeaponType(IntEnum):
    RIFLE = 0
    DMR = 1
    SNIPER_RIFLE = 2
    LIGHT_SUPPORT_GUN = 3
    LIGHT_MACHINE_GUN = 4
    SUB_MACHINE_GUN = 5
    PISTOL = 6
    AUTO_PISTOL = 7
    HEAVY_PISTOL = 8
    CARBINE = 9
    PERSONAL_DEFENSE_WEAPON = 10
